@@ -11,6 +11,9 @@ import py_cui
 import py_cui.errors
 import py_cui.colors
 
+import inspect
+from typing import Any, List, Optional, Tuple, Callable
+
 
 class UIElement:
     """Base class for all UI elements. Extended by base widget and popup classes.
@@ -67,28 +70,27 @@ class UIElement:
         self._border_color              = self._color
         self._focus_border_color        = self._color
         self._selected_color            = self._color
-        self._mouse_press_handler       = None
         self._selected                  = False
         self._renderer                  = renderer
         self._logger                    = logger
         self._help_text                 = ''
 
 
-    def get_absolute_start_pos(self):
+    def get_absolute_start_pos(self) -> Tuple[int,int]:
         """Must be implemented by subclass, computes the absolute coords of upper-left corner
         """
 
         raise NotImplementedError
 
 
-    def get_absolute_stop_pos(self):
+    def get_absolute_stop_pos(self) -> Tuple[int,int]:
         """Must be implemented by subclass, computes the absolute coords of bottom-right corner
         """
 
         raise NotImplementedError
 
 
-    def get_absolute_dimensions(self):
+    def get_absolute_dimensions(self) -> Tuple[int,int]:
         """Gets dimensions of element in terminal characters
 
         Returns
@@ -101,7 +103,7 @@ class UIElement:
         return (stop_y - start_y), (stop_x - start_x)
 
 
-    def update_height_width(self):
+    def update_height_width(self) -> None:
         """Function that refreshes position and dimensons on resize.
 
         If necessary, make sure required widget attributes updated here as well.
@@ -112,7 +114,7 @@ class UIElement:
         self._height,  self._width    = self.get_absolute_dimensions()
 
 
-    def get_viewport_height(self):
+    def get_viewport_height(self) -> int:
         """Gets the height of the element viewport (height minus padding and borders)
 
         Returns
@@ -124,7 +126,7 @@ class UIElement:
         return self._height - (2 * self._pady) - 3
 
 
-    def get_id(self):
+    def get_id(self) -> int:
         """Gets the element ID
 
         Returns
@@ -136,7 +138,7 @@ class UIElement:
         return self._id
 
 
-    def get_title(self):
+    def get_title(self) -> str:
         """Getter for ui element title
 
         Returns
@@ -148,7 +150,7 @@ class UIElement:
         return self._title
 
 
-    def get_padding(self):
+    def get_padding(self) -> Tuple[int,int]:
         """Gets ui element padding on in characters
 
         Returns
@@ -160,7 +162,7 @@ class UIElement:
         return self._padx, self._pady
 
 
-    def get_start_position(self):
+    def get_start_position(self) -> Tuple[int,int]:
         """Gets coords of upper left corner
 
         Returns
@@ -172,7 +174,7 @@ class UIElement:
         return self._start_x, self._start_y
 
 
-    def get_stop_position(self):
+    def get_stop_position(self) -> Tuple[int,int]:
         """Gets coords of lower right corner
 
         Returns
@@ -184,7 +186,7 @@ class UIElement:
         return self._stop_x, self._stop_y
 
 
-    def get_color(self):
+    def get_color(self) -> int:
         """Gets current element color
 
         Returns
@@ -196,7 +198,7 @@ class UIElement:
         return self._color
 
 
-    def get_border_color(self):
+    def get_border_color(self) -> int:
         """Gets current element border color
 
         Returns
@@ -211,7 +213,7 @@ class UIElement:
             return self._border_color
 
 
-    def get_selected_color(self):
+    def get_selected_color(self) -> int:
         """Gets current selected item color
 
         Returns
@@ -223,7 +225,7 @@ class UIElement:
         return self._selected_color
 
 
-    def is_selected(self):
+    def is_selected(self) -> bool:
         """Get selected status
 
         Returns
@@ -235,7 +237,7 @@ class UIElement:
         return self._selected
 
 
-    def get_renderer(self):
+    def get_renderer(self) -> 'py_cui.renderer.Renderer':
         """Gets reference to renderer object
 
         Returns
@@ -247,7 +249,7 @@ class UIElement:
         return self._renderer
 
 
-    def get_help_text(self):
+    def get_help_text(self) -> str:
         """Returns current help text
 
         Returns
@@ -259,7 +261,7 @@ class UIElement:
         return self._help_text
 
 
-    def set_title(self, title):
+    def set_title(self, title: str):
         """Function that sets the widget title.
 
         Parameters
@@ -271,7 +273,7 @@ class UIElement:
         self._title = title
 
 
-    def set_color(self, color):
+    def set_color(self, color: int) -> None:
         """Sets element default color
 
         Parameters
@@ -289,7 +291,7 @@ class UIElement:
         self._color = color
 
 
-    def set_border_color(self, color):
+    def set_border_color(self, color: int) -> None:
         """Sets element border color
 
         Parameters
@@ -301,7 +303,7 @@ class UIElement:
         self._border_color = color
 
 
-    def set_focus_border_color(self, color):
+    def set_focus_border_color(self, color: int) -> None:
         """Sets element border color if the current element
         is focused
 
@@ -314,7 +316,7 @@ class UIElement:
         self._focus_border_color = color
 
 
-    def set_selected_color(self, color):
+    def set_selected_color(self, color: int) -> None:
         """Sets element sected color
 
         Parameters
@@ -326,7 +328,7 @@ class UIElement:
         self._selected_color = color
 
 
-    def set_selected(self, selected):
+    def set_selected(self, selected: bool) -> None:
         """Marks the UI element as selected or not selected
 
         Parameters
@@ -338,7 +340,7 @@ class UIElement:
         self._selected = selected
 
 
-    def set_help_text(self, help_text):
+    def set_help_text(self, help_text: str) -> None:
         """Sets status bar help text
 
         Parameters
@@ -350,7 +352,7 @@ class UIElement:
         self._help_text = help_text
 
 
-    def set_focus_text(self, focus_text):
+    def set_focus_text(self, focus_text: str) -> None:
         """Sets status bar focus text. Legacy function, overridden by set_focus_text
 
         Parameters
@@ -369,19 +371,7 @@ class UIElement:
         raise NotImplementedError
 
 
-    def add_mouse_press_handler(self, mouse_press_handler_func):
-        """Sets a mouse press handler function
-
-        Parameters
-        ----------
-        mouse_press_handler_func : function / lambda function
-            Function that takes 2 parameters: x and y of a mouse press. Executes when mouse pressed and element is selected
-        """
-
-        self._mouse_press_handler = mouse_press_handler_func
-
-
-    def _handle_mouse_press(self, x, y):
+    def _handle_mouse_press(self, x, y, mouse_event):
         """Can be implemented by subclass. Used to handle mouse presses
 
         Parameters
@@ -390,8 +380,7 @@ class UIElement:
             Coordinates of the mouse press event.
         """
 
-        if self._mouse_press_handler is not None:
-            self._mouse_press_handler(x, y)
+        pass
 
 
     def _draw(self):
@@ -401,7 +390,7 @@ class UIElement:
         raise NotImplementedError
 
 
-    def _assign_renderer(self, renderer):
+    def _assign_renderer(self, renderer: 'py_cui.renderer.Renderer', quiet: bool=False) :
         """Function that assigns a renderer object to the element
 
         (Meant for internal usage only)
@@ -417,15 +406,17 @@ class UIElement:
             If parameter is not an initialized renderer.
         """
 
-        if isinstance(renderer, py_cui.renderer.Renderer):
-            self._renderer = renderer
+        if renderer is None:
+            self._logger.debug('Renderer to assign is a NoneType')
         elif self._renderer is not None:
             raise py_cui.errors.PyCUIError('Renderer already assigned for the element')
+        elif isinstance(renderer, py_cui.renderer.Renderer):
+            self._renderer = renderer
         else:
             raise py_cui.errors.PyCUIError('Invalid renderer, must be of type py_cui.renderer.Renderer')
 
 
-    def _contains_position(self, x, y):
+    def _contains_position(self, x: int, y: int) -> bool:
         """Checks if character position is within element.
 
         Parameters
@@ -484,7 +475,7 @@ class TextBoxImplementation(UIImplementation):
         Toggle to display password characters or text
     """
 
-    def __init__(self, initial_text, password, logger):
+    def __init__(self, initial_text: str, password: bool , logger):
         """Initializer for the TextBoxImplementation base class
         """
 
@@ -501,7 +492,7 @@ class TextBoxImplementation(UIImplementation):
 
     # Variable getter + setter functions
 
-    def get_initial_cursor_pos(self):
+    def get_initial_cursor_pos(self) -> int:
         """Gets initial cursor position
 
         Returns
@@ -513,7 +504,7 @@ class TextBoxImplementation(UIImplementation):
         return self._initial_cursor
 
 
-    def get_cursor_text_pos(self):
+    def get_cursor_text_pos(self) -> int:
         """Gets current position of cursor relative to text
 
         Returns
@@ -525,7 +516,7 @@ class TextBoxImplementation(UIImplementation):
         return self._cursor_text_pos
 
 
-    def get_cursor_limits(self):
+    def get_cursor_limits(self) -> Tuple[int,int]:
         """Gets cursor extreme points in terminal position
 
         Returns
@@ -537,7 +528,7 @@ class TextBoxImplementation(UIImplementation):
         return self._cursor_max_left, self._cursor_max_right
 
 
-    def get_cursor_position(self):
+    def get_cursor_position(self) -> Tuple[int,int]:
         """Returns current cursor poition
 
         Returns
@@ -549,7 +540,7 @@ class TextBoxImplementation(UIImplementation):
         return self._cursor_x, self._cursor_y
 
 
-    def get_viewport_width(self):
+    def get_viewport_width(self) -> int:
         """Gets the width of the textbox viewport
 
         Returns
@@ -561,7 +552,7 @@ class TextBoxImplementation(UIImplementation):
         return self._viewport_width
 
 
-    def set_text(self, text):
+    def set_text(self, text: str):
         """Sets the value of the text. Overwrites existing text
 
         Parameters
@@ -577,7 +568,7 @@ class TextBoxImplementation(UIImplementation):
             self._cursor_x = self._cursor_x - diff
 
 
-    def get(self):
+    def get(self) -> str:
         """Gets value of the text in the textbox
 
         Returns
@@ -589,7 +580,7 @@ class TextBoxImplementation(UIImplementation):
         return self._text
 
 
-    def clear(self):
+    def clear(self) -> None:
         """Clears the text in the textbox
         """
 
@@ -598,7 +589,7 @@ class TextBoxImplementation(UIImplementation):
         self._text             = ''
 
 
-    def _move_left(self):
+    def _move_left(self) -> None:
         """Shifts the cursor the the left. Internal use only
         """
 
@@ -608,7 +599,7 @@ class TextBoxImplementation(UIImplementation):
             self._cursor_text_pos = self._cursor_text_pos - 1
 
 
-    def _move_right(self):
+    def _move_right(self) -> None:
         """Shifts the cursor the the right. Internal use only
         """
         if self._cursor_text_pos < len(self._text):
@@ -617,7 +608,7 @@ class TextBoxImplementation(UIImplementation):
             self._cursor_text_pos = self._cursor_text_pos + 1
 
 
-    def _insert_char(self, key_pressed):
+    def _insert_char(self, key_pressed: int) -> None:
         """Inserts char at cursor position. Internal use only
 
         Parameters
@@ -631,7 +622,7 @@ class TextBoxImplementation(UIImplementation):
         self._cursor_text_pos = self._cursor_text_pos + 1
 
 
-    def _jump_to_start(self):
+    def _jump_to_start(self) -> None:
         """Jumps to the start of the textbox. Internal use only
         """
 
@@ -639,7 +630,7 @@ class TextBoxImplementation(UIImplementation):
         self._cursor_text_pos = 0
 
 
-    def _jump_to_end(self):
+    def _jump_to_end(self) -> None:
         """Jumps to the end to the textbox. Internal use only
         """
 
@@ -647,7 +638,7 @@ class TextBoxImplementation(UIImplementation):
         self._cursor_x = self._initial_cursor + self._cursor_text_pos
 
 
-    def _erase_char(self):
+    def _erase_char(self) -> None:
         """Erases character at textbox cursor. Internal Use only
         """
 
@@ -658,7 +649,7 @@ class TextBoxImplementation(UIImplementation):
             self._cursor_text_pos = self._cursor_text_pos - 1
 
 
-    def _delete_char(self):
+    def _delete_char(self) -> None:
         """Deletes character to right of texbox cursor. Internal use only
         """
 
@@ -691,9 +682,10 @@ class MenuImplementation(UIImplementation):
         self._selected_item    = 0
         self._page_scroll_len  = 5
         self._view_items       = []
+        self._on_selection_change: Optional[Callable[[Any],Any]] = None
 
 
-    def clear(self):
+    def clear(self) -> None:
         """Clears all items from the Scroll Menu
         """
 
@@ -704,8 +696,53 @@ class MenuImplementation(UIImplementation):
         self._logger.info('Clearing menu')
 
 
+    def set_on_selection_change_event(self, on_selection_change_event: Callable[[Any],Any]):
+        """Function that sets the function fired when menu selection changes.
 
-    def get_selected_item_index(self):
+        Event function must take 0 or 1 parameters. If 1 parameter, the new selcted item will be passed in.
+
+        Parameters
+        ----------
+        on_selection_change_event : Callable
+            Callable function that takes in as an argument the newly selected element
+
+        Raises
+        ------
+        TypeError
+            Raises a type error if event function is not callable
+        """
+
+        #mypy false-positive
+        if not isinstance(on_selection_change_event, Callable):  #type: ignore
+            raise TypeError('On selection change event must be a Callable!')
+        
+        self._on_selection_change = on_selection_change_event
+
+
+    def _process_selection_change_event(self):
+        """Function that executes on-selection change event either with the current menu item, or with no-args"""
+
+        # Identify num of args from callable. This allows for user to create commands that take in x, y
+        # coords of the mouse press as input
+        num_args = 0
+        try:
+            num_args = len(inspect.signature(self._on_selection_change).parameters)
+        except ValueError:
+            self._logger.error('Failed to get on_selection_change signature!')
+        except TypeError:
+            self._logger.error('Type of object not supported for signature identification!')
+
+        # Depending on the number of parameters for the self._on_selection_change, pass in the x and y
+        # values, or do nothing
+        if num_args == 1:
+            self._on_selection_change(self.get())
+        elif num_args == 0:
+            self._on_selection_change()
+        else:
+            raise ValueError('On selection change event must accept either 0 or 1 parameters!')
+
+
+    def get_selected_item_index(self) -> int:
         """Gets the currently selected item
 
         Returns
@@ -717,7 +754,7 @@ class MenuImplementation(UIImplementation):
         return self._selected_item
 
 
-    def set_selected_item_index(self, selected_item_index):
+    def set_selected_item_index(self, selected_item_index: int) -> None:
         """Sets the currently selected item
 
         Parameters
@@ -729,7 +766,7 @@ class MenuImplementation(UIImplementation):
         self._selected_item = selected_item_index
 
 
-    def _scroll_up(self):
+    def _scroll_up(self) -> None:
         """Function that scrolls the view up in the scroll menu
         """
 
@@ -738,10 +775,10 @@ class MenuImplementation(UIImplementation):
         if self._selected_item > 0:
             self._selected_item = self._selected_item - 1
 
-        self._logger.info('Scrolling up to item {}'.format(self._selected_item))
+        self._logger.debug(f'Scrolling up to item {self._selected_item}')
 
 
-    def _scroll_down(self, viewport_height):
+    def _scroll_down(self, viewport_height: int) -> None:
         """Function that scrolls the view down in the scroll menu
 
         TODO: Viewport height should be calculated internally, and not rely on a parameter.
@@ -757,10 +794,10 @@ class MenuImplementation(UIImplementation):
         if self._selected_item > self._top_view + viewport_height:
             self._top_view = self._top_view + 1
 
-        self._logger.info('Scrolling down to item {}'.format(self._selected_item))
+        self._logger.debug(f'Scrolling down to item {self._selected_item}')
 
 
-    def _jump_up(self):
+    def _jump_up(self) -> None:
         """Function for jumping up menu several spots at a time
         """
 
@@ -768,7 +805,7 @@ class MenuImplementation(UIImplementation):
             self._scroll_up()
 
 
-    def _jump_down(self, viewport_height):
+    def _jump_down(self, viewport_height: int) -> None:
         """Function for jumping down the menu several spots at a time
 
         Parameters
@@ -781,7 +818,7 @@ class MenuImplementation(UIImplementation):
             self._scroll_down(viewport_height)
 
 
-    def _jump_to_top(self):
+    def _jump_to_top(self) -> None:
         """Function that jumps to the top of the menu
         """
 
@@ -789,7 +826,7 @@ class MenuImplementation(UIImplementation):
         self._selected_item = 0
 
 
-    def _jump_to_bottom(self, viewport_height):
+    def _jump_to_bottom(self, viewport_height: int) -> None:
         """Function that jumps to the bottom of the menu
 
         Parameters
@@ -804,7 +841,7 @@ class MenuImplementation(UIImplementation):
             self._top_view = 0
 
 
-    def add_item(self, item):
+    def add_item(self, item: Any) -> None: 
         """Adds an item to the menu.
 
         Parameters
@@ -813,11 +850,12 @@ class MenuImplementation(UIImplementation):
             Object to add to the menu. Must have implemented __str__ function
         """
 
-        self._logger.info('Adding item {} to menu'.format(str(item)))
+        self._logger.debug(f'Adding item {str(item)} to menu')
         self._view_items.append(item)
 
 
-    def add_item_list(self, item_list):
+    def add_item_list(self, item_list: List[Any]) -> None:
+
         """Adds a list of items to the scroll menu.
 
         Parameters
@@ -826,24 +864,24 @@ class MenuImplementation(UIImplementation):
             list of objects to add as items to the scrollmenu
         """
 
-        self._logger.info('Adding item list {} to menu'.format(str(item_list)))
+        self._logger.debug(f'Adding item list {str(item_list)} to menu')
         for item in item_list:
             self.add_item(item)
 
 
-    def remove_selected_item(self):
+    def remove_selected_item(self) -> None:
         """Function that removes the selected item from the scroll menu.
         """
 
         if len(self._view_items) == 0:
             return
-        self._logger.info('Removing {}'.format(str(self._view_items[self._selected_item])))
+        self._logger.debug(f'Removing {str(self._view_items[self._selected_item])}')
         del self._view_items[self._selected_item]
         if self._selected_item >= len(self._view_items) and self._selected_item > 0:
             self._selected_item = self._selected_item - 1
 
 
-    def remove_item(self, item):
+    def remove_item(self, item: Any) -> None:
         """Function that removes a specific item from the menu
 
         Parameters
@@ -854,14 +892,14 @@ class MenuImplementation(UIImplementation):
 
         if len(self._view_items) == 0 or item not in self._view_items:
             return
-        self._logger.info('Removing {}'.format(str(item)))
+        self._logger.debug(f'Removing {str(item)}')
         i_index = self._view_items.index(item)
         del self._view_items[i_index]
         if self._selected_item >= i_index:
             self._selected_item = self._selected_item - 1
 
 
-    def get_item_list(self):
+    def get_item_list(self) -> List[Any]:
         """Function that gets list of items in a scroll menu
 
         Returns
@@ -873,7 +911,7 @@ class MenuImplementation(UIImplementation):
         return self._view_items
 
 
-    def get(self):
+    def get(self) -> Optional[Any]:
         """Function that gets the selected item from the scroll menu
 
         Returns
@@ -887,7 +925,7 @@ class MenuImplementation(UIImplementation):
         return None
 
 
-    def set_selected_item(self, selected_item):
+    def set_selected_item(self, selected_item: Any):
         """Function that replaces the currently selected item with a new item
 
         Parameters
@@ -920,7 +958,7 @@ class CheckBoxMenuImplementation(MenuImplementation):
         self._checked_char       = checked_char
 
 
-    def add_item(self, item):
+    def add_item(self, item: Any) -> None:
         """Extends base class function, item is added and marked as unchecked to start
 
         Parameters
@@ -933,7 +971,7 @@ class CheckBoxMenuImplementation(MenuImplementation):
         self._selected_item_dict[item] = False
 
 
-    def remove_selected_item(self):
+    def remove_selected_item(self) -> None:
         """Removes selected item from item list and selected item dictionary
         """
 
@@ -941,7 +979,7 @@ class CheckBoxMenuImplementation(MenuImplementation):
         super().remove_selected_item()
 
 
-    def remove_item(self, item):
+    def remove_item(self, item) -> None:
         """Removes item from item list and selected item dict
 
         Parameters
@@ -954,18 +992,42 @@ class CheckBoxMenuImplementation(MenuImplementation):
         super().remove_item(item)
 
 
-    def mark_item_as_checked(self, item):
+    def toggle_item_checked(self, item: Any):
         """Function that marks an item as selected
 
         Parameters
         ----------
         item : object
-            Mark item as checked
+            Toggle item checked state
         """
 
         self._selected_item_dict[item] = not self._selected_item_dict[item]
 
 
+    def mark_item_as_checked(self, item: Any) -> None:
+        """Function that marks an item as selected
+
+        Parameters
+        ----------
+        item : object
+            Toggle item checked state
+        """
+
+        self._selected_item_dict[item] = True
+
+
+    def mark_item_as_not_checked(self, item) -> None:
+        """Function that marks an item as selected
+
+        Parameters
+        ----------
+        item : object
+            Item to uncheck
+        """
+
+        self._selected_item_dict[item] = False
+
+
 class TextBlockImplementation(UIImplementation):
     """Base class for TextBlockImplementation
 
@@ -990,7 +1052,7 @@ class TextBlockImplementation(UIImplementation):
         The dimensions of the viewport in characters
     """
 
-    def __init__(self, initial_text, logger):
+    def __init__(self, initial_text: str, logger):
         """Initializer for TextBlockImplementation base class
 
         Zeros attributes, and parses initial text
@@ -1017,7 +1079,7 @@ class TextBlockImplementation(UIImplementation):
 
     # Getters and setters
 
-    def get_viewport_start_pos(self):
+    def get_viewport_start_pos(self) -> Tuple[int,int]:
         """Gets upper left corner position of viewport
 
         Returns
@@ -1029,7 +1091,7 @@ class TextBlockImplementation(UIImplementation):
         return self._viewport_x_start, self._viewport_y_start
 
 
-    def get_viewport_dims(self):
+    def get_viewport_dims(self) -> Tuple[int,int]:
         """Gets viewport dimensions in characters
 
         Returns
@@ -1041,7 +1103,7 @@ class TextBlockImplementation(UIImplementation):
         return self._viewport_height, self._viewport_width
 
 
-    def get_cursor_text_pos(self):
+    def get_cursor_text_pos(self) -> Tuple[int,int]:
         """Gets cursor postion relative to text
 
         Returns
@@ -1054,7 +1116,7 @@ class TextBlockImplementation(UIImplementation):
         return self._cursor_text_pos_x, self._cursor_text_pos_y
 
 
-    def get_abs_cursor_position(self):
+    def get_abs_cursor_position(self) -> Tuple[int,int]:
         """Gets absolute cursor position in terminal characters
 
         Returns
@@ -1066,7 +1128,7 @@ class TextBlockImplementation(UIImplementation):
         return self._cursor_x, self._cursor_y
 
 
-    def get_cursor_limits_vertical(self):
+    def get_cursor_limits_vertical(self) -> Tuple[int,int]:
         """Gets limits for cursor in vertical direction
 
         Returns
@@ -1078,7 +1140,7 @@ class TextBlockImplementation(UIImplementation):
         return self._cursor_max_up, self._cursor_max_down
 
 
-    def get_cursor_limits_horizontal(self):
+    def get_cursor_limits_horizontal(self) -> Tuple[int,int]:
         """Gets limits for cursor in horizontal direction
 
         Returns
@@ -1090,7 +1152,7 @@ class TextBlockImplementation(UIImplementation):
         return self._cursor_max_left, self._cursor_max_right
 
 
-    def get(self):
+    def get(self) -> str:
         """Gets all of the text in the textblock and returns it
 
         Returns
@@ -1101,11 +1163,11 @@ class TextBlockImplementation(UIImplementation):
 
         text = ''
         for line in self._text_lines:
-            text = '{}{}\n'.format(text, line)
+            text = f'{text}{line}\n'
         return text
 
 
-    def write(self, text):
+    def write(self, text: str) -> None:
         """Function used for writing text to the text block
 
         Parameters
@@ -1118,10 +1180,10 @@ class TextBlockImplementation(UIImplementation):
         if len(self._text_lines) == 1 and self._text_lines[0] == '':
             self.set_text(text)
         else:
-            self._text_lines.append(lines)
+            self._text_lines.extend(lines)
 
 
-    def clear(self):
+    def clear(self) -> None:
         """Function that clears the text block
         """
 
@@ -1134,7 +1196,7 @@ class TextBlockImplementation(UIImplementation):
         self._logger.info('Cleared textblock')
 
 
-    def get_current_line(self):
+    def get_current_line(self) -> str:
         """Returns the line on which the cursor currently resides
 
         Returns
@@ -1146,7 +1208,7 @@ class TextBlockImplementation(UIImplementation):
         return self._text_lines[self._cursor_text_pos_y]
 
 
-    def set_text(self, text):
+    def set_text(self, text: str) -> None:
         """Function that sets the text for the textblock.
 
         Note that this will overwrite any existing text
@@ -1168,7 +1230,7 @@ class TextBlockImplementation(UIImplementation):
         self._cursor_text_pos_x    = 0
 
 
-    def set_text_line(self, text):
+    def set_text_line(self, text: str) -> None:
         """Function that sets the current line's text.
 
         Meant only for internal use
@@ -1182,7 +1244,7 @@ class TextBlockImplementation(UIImplementation):
         self._text_lines[self._cursor_text_pos_y] = text
 
 
-    def _move_left(self):
+    def _move_left(self) -> None:
         """Function that moves the cursor/text position one location to the left
         """
 
@@ -1193,10 +1255,10 @@ class TextBlockImplementation(UIImplementation):
                 self._viewport_x_start = self._viewport_x_start - 1
             self._cursor_text_pos_x = self._cursor_text_pos_x - 1
 
-        self._logger.info('Moved cursor left to pos {}'.format(self._cursor_text_pos_x))
+        self._logger.debug(f'Moved cursor left to pos {self._cursor_text_pos_x}')
 
 
-    def _move_right(self):
+    def _move_right(self) -> None:
         """Function that moves the cursor/text position one location to the right
         """
 
@@ -1209,10 +1271,10 @@ class TextBlockImplementation(UIImplementation):
                 self._viewport_x_start = self._viewport_x_start + 1
             self._cursor_text_pos_x = self._cursor_text_pos_x + 1
 
-        self._logger.info('Moved cursor right to pos {}'.format(self._cursor_text_pos_x))
+        self._logger.debug(f'Moved cursor right to pos {self._cursor_text_pos_x}')
 
 
-    def _move_up(self):
+    def _move_up(self) -> None:
         """Function that moves the cursor/text position one location up
         """
 
@@ -1228,10 +1290,10 @@ class TextBlockImplementation(UIImplementation):
                 self._cursor_x = self._cursor_x - (self._cursor_text_pos_x - temp)
                 self._cursor_text_pos_x = temp
 
-        self._logger.info('Moved cursor up to line {}'.format(self._cursor_text_pos_y))
+        self._logger.debug(f'Moved cursor up to line {self._cursor_text_pos_y}')
 
 
-    def _move_down(self):
+    def _move_down(self) -> None:
         """Function that moves the cursor/text position one location down
         """
 
@@ -1246,16 +1308,16 @@ class TextBlockImplementation(UIImplementation):
                 self._cursor_x = self._cursor_x - (self._cursor_text_pos_x - temp)
                 self._cursor_text_pos_x = temp
 
-        self._logger.info('Moved cursor down to line {}'.format(self._cursor_text_pos_y))
+        self._logger.debug(f'Moved cursor down to line {self._cursor_text_pos_y}')
 
 
 
-    def _handle_newline(self):
+    def _handle_newline(self) -> None:
         """Function that handles recieving newline characters in the text
         """
 
         current_line = self.get_current_line()
-        self._logger.info('Inserting newline in location {}'.format(self._cursor_text_pos_x))
+        self._logger.debug(f'Inserting newline in location {self._cursor_text_pos_x}')
 
         new_line_1 = current_line[:self._cursor_text_pos_x]
         new_line_2 = current_line[self._cursor_text_pos_x:]
@@ -1271,12 +1333,12 @@ class TextBlockImplementation(UIImplementation):
             self._viewport_y_start = self._viewport_y_start + 1
 
 
-    def _handle_backspace(self):
+    def _handle_backspace(self) -> None:
         """Function that handles recieving backspace characters in the text
         """
 
         current_line = self.get_current_line()
-        self._logger.info('Inserting backspace in location {}'.format(self._cursor_text_pos_x))
+        self._logger.debug(f'Inserting backspace in location {self._cursor_text_pos_x}')
 
         if self._cursor_text_pos_x == 0 and self._cursor_text_pos_y != 0:
             self._cursor_text_pos_x = len(self._text_lines[self._cursor_text_pos_y - 1])
@@ -1295,23 +1357,23 @@ class TextBlockImplementation(UIImplementation):
             self._cursor_text_pos_x = self._cursor_text_pos_x - 1
 
 
-    def _handle_home(self):
+    def _handle_home(self) -> None:
         """Function that handles recieving a home keypress
         """
 
-        self._logger.info('Inserting Home')
+        self._logger.debug('Inserting Home')
 
         self._cursor_x = self._cursor_max_left
         self._cursor_text_pos_x = 0
         self._viewport_x_start = 0
 
 
-    def _handle_end(self):
+    def _handle_end(self) -> None:
         """Function that handles recieving an end keypress
         """
 
         current_line = self.get_current_line()
-        self._logger.info('Inserting End')
+        self._logger.debug('Inserting End')
 
         self._cursor_text_pos_x = len(current_line)
         if len(current_line) > self._viewport_width:
@@ -1321,12 +1383,12 @@ class TextBlockImplementation(UIImplementation):
             self._cursor_x = self._cursor_max_left + len(current_line)
 
 
-    def _handle_delete(self):
+    def _handle_delete(self) -> None:
         """Function that handles recieving a delete keypress
         """
 
         current_line = self.get_current_line()
-        self._logger.info('Inserting delete to pos {}'.format(self._cursor_text_pos_x))
+        self._logger.debug(f'Inserting delete to pos {self._cursor_text_pos_x}')
 
         if self._cursor_text_pos_x == len(current_line) and self._cursor_text_pos_y < len(self._text_lines) - 1:
             self._text_lines[self._cursor_text_pos_y] = self._text_lines[self._cursor_text_pos_y] + self._text_lines[self._cursor_text_pos_y + 1]
@@ -1335,7 +1397,7 @@ class TextBlockImplementation(UIImplementation):
             self.set_text_line(current_line[:self._cursor_text_pos_x] + current_line[self._cursor_text_pos_x+1:])
 
 
-    def _insert_char(self, key_pressed):
+    def _insert_char(self, key_pressed: int) -> None:
         """Function that handles recieving a character
 
         Parameters
@@ -1345,7 +1407,7 @@ class TextBlockImplementation(UIImplementation):
         """
 
         current_line = self.get_current_line()
-        self._logger.info('Inserting character {} to pos {}'.format(chr(key_pressed), self._cursor_text_pos_x))
+        self._logger.debug(f'Inserting character {chr(key_pressed)} to pos {self._cursor_text_pos_x}')
 
         self.set_text_line(current_line[:self._cursor_text_pos_x] + chr(key_pressed) + current_line[self._cursor_text_pos_x:])
         if len(current_line) <= self._viewport_width:
