@@ -9,10 +9,14 @@ It can be used to swap between collections of widgets (screens) in a py_cui
 # TODO: Should create an initial widget set in PyCUI class that widgets are added to by default.
 
 import shutil
+from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING
 import py_cui.widgets as widgets
 import py_cui.grid as grid
 import py_cui.controls as controls
 
+if TYPE_CHECKING:
+    import py_cui
+
 
 class WidgetSet:
     """Class that represents a collection of widgets.
@@ -29,15 +33,18 @@ class WidgetSet:
         list of keybindings to check against in the main CUI loop
     height, width : int
         height of the terminal in characters, width of terminal in characters
+    root : py_cui.PyCUI
+        Main PyCUI object reference
     """
 
-    def __init__(self, num_rows, num_cols, logger, simulated_terminal=None):
+    def __init__(self, num_rows: int, num_cols: int, logger: 'py_cui.debug.PyCUILogger', root:'py_cui.PyCUI', simulated_terminal: Optional[List[int]] =None):
         """Constructor for WidgetSet
         """
 
-        self._widgets      = {}
-        self._keybindings  = {}
+        self._widgets: Dict[int,Optional['py_cui.widgets.Widget']]      = {}
+        self._keybindings: Dict[int,Callable[[],Any]]  = {}
 
+        self._root = root
         self._simulated_terminal = simulated_terminal
 
         if self._simulated_terminal is None:
@@ -50,15 +57,16 @@ class WidgetSet:
 
         self._height = height
         self._width = width
-        self._height = self._height - 4
+        status_bars_height = self._root.title_bar.get_height() + self._root.status_bar.get_height()
+        self._height = self._height - status_bars_height - 2
 
         self._grid = grid.Grid(num_rows, num_cols, self._height, self._width, logger)
 
-        self._selected_widget = None
+        self._selected_widget: Optional[int] = None
         self._logger = logger
 
 
-    def set_selected_widget(self, widget_id):
+    def set_selected_widget(self, widget_id: int) -> None:
         """Function that sets the selected cell for the CUI
 
         Parameters
@@ -71,7 +79,7 @@ class WidgetSet:
             self._selected_widget = widget_id
 
 
-    def get_widgets(self):
+    def get_widgets(self) -> Dict[int, Optional['py_cui.widgets.Widget']]:
         """Function that gets current set of widgets
 
         Returns
@@ -83,7 +91,7 @@ class WidgetSet:
         return self._widgets
 
 
-    def add_key_command(self, key, command):
+    def add_key_command(self, key: int, command: Callable[[],Any]):
         """Function that adds a keybinding to the CUI when in overview mode
 
         Parameters
@@ -97,7 +105,7 @@ class WidgetSet:
         self._keybindings[key] = command
 
 
-    def add_scroll_menu(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0):
+    def add_scroll_menu(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0) -> 'py_cui.widgets.ScrollMenu':
         """Function that adds a new scroll menu to the CUI grid
 
         Parameters
@@ -123,7 +131,7 @@ class WidgetSet:
             A reference to the created scroll menu object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_scroll_menu     = widgets.ScrollMenu(id,
                                                  title,
                                                  self._grid,
@@ -137,11 +145,11 @@ class WidgetSet:
         self._widgets[id]  = new_scroll_menu
         if self._selected_widget is None:
             self.set_selected_widget(id)
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_scroll_menu))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_scroll_menu))}')
         return new_scroll_menu
 
 
-    def add_checkbox_menu(self, title, row, column, row_span=1, column_span=1, padx=1, pady=0, checked_char='X'):
+    def add_checkbox_menu(self, title: str, row: int, column: int, row_span: int=1, column_span: int=1, padx: int=1, pady: int=0, checked_char: str='X') -> 'py_cui.widgets.CheckBoxMenu':
         """Function that adds a new checkbox menu to the CUI grid
 
         Parameters
@@ -169,7 +177,7 @@ class WidgetSet:
             A reference to the created checkbox object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_checkbox_menu   = widgets.CheckBoxMenu(id,
                                                    title,
                                                    self._grid,
@@ -184,11 +192,11 @@ class WidgetSet:
         self._widgets[id]  = new_checkbox_menu
         if self._selected_widget is None:
             self.set_selected_widget(id)
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_checkbox_menu))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_checkbox_menu))}')
         return new_checkbox_menu
 
 
-    def add_text_box(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0, initial_text = '', password = False):
+    def add_text_box(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0, initial_text: str = '', password: bool = False) -> 'py_cui.widgets.TextBox':
         """Function that adds a new text box to the CUI grid
 
         Parameters
@@ -218,7 +226,7 @@ class WidgetSet:
             A reference to the created textbox object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_text_box        = widgets.TextBox(id,
                                               title,
                                               self._grid,
@@ -232,11 +240,11 @@ class WidgetSet:
         self._widgets[id]    = new_text_box
         if self._selected_widget is None:
             self.set_selected_widget(id)
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_text_box))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_text_box))}')
         return new_text_box
 
 
-    def add_text_block(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0, initial_text = ''):
+    def add_text_block(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0, initial_text: str = '') -> 'py_cui.widgets.ScrollTextBlock':
         """Function that adds a new text block to the CUI grid
 
         Parameters
@@ -264,7 +272,7 @@ class WidgetSet:
             A reference to the created textblock object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_text_block      = widgets.ScrollTextBlock(id,
                                                       title,
                                                       self._grid,
@@ -279,11 +287,11 @@ class WidgetSet:
         self._widgets[id]  = new_text_block
         if self._selected_widget is None:
             self.set_selected_widget(id)
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_text_block))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_text_block))}')
         return new_text_block
 
 
-    def add_label(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0):
+    def add_label(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0) -> 'py_cui.widgets.Label':
         """Function that adds a new label to the CUI grid
 
         Parameters
@@ -309,7 +317,7 @@ class WidgetSet:
             A reference to the created label object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_label           = widgets.Label(id,
                                             title,
                                             self._grid,
@@ -321,11 +329,11 @@ class WidgetSet:
                                             pady,
                                             self._logger)
         self._widgets[id]  = new_label
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_label))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_label))}')
         return new_label
 
 
-    def add_block_label(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0, center=True):
+    def add_block_label(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0, center: bool=True) -> 'py_cui.widgets.BlockLabel':
         """Function that adds a new block label to the CUI grid
 
         Parameters
@@ -353,7 +361,7 @@ class WidgetSet:
             A reference to the created block label object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_label           = widgets.BlockLabel(id,
                                                  title,
                                                  self._grid,
@@ -366,11 +374,11 @@ class WidgetSet:
                                                  center,
                                                  self._logger)
         self._widgets[id]  = new_label
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_label))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_label))}')
         return new_label
 
 
-    def add_button(self, title, row, column, row_span = 1, column_span = 1, padx = 1, pady = 0, command=None):
+    def add_button(self, title: str, row: int, column: int, row_span: int = 1, column_span: int = 1, padx: int = 1, pady: int = 0, command: Optional[Callable[[],Any]]=None) -> 'py_cui.widgets.Button':
         """Function that adds a new button to the CUI grid
 
         Parameters
@@ -398,7 +406,7 @@ class WidgetSet:
             A reference to the created button object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self.get_widgets().keys())
         new_button          = widgets.Button(id,
                                              title,
                                              self._grid,
@@ -413,13 +421,15 @@ class WidgetSet:
         self._widgets[id]  = new_button
         if self._selected_widget is None:
             self.set_selected_widget(id)
-        self._logger.info('Adding widget {} w/ ID {} of type {}'.format(title, id, str(type(new_button))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_button))}')
         return new_button
 
 
-    def add_slider(self, title, row, column, row_span=1,
-                   column_span=1, padx=1, pady=0,
-                   min_val=0, max_val=100, step=1, init_val=0):
+    def add_slider(self, title: str, row: int, column: int, row_span: int=1,
+                   column_span: int=1, padx: int=1, pady: int=0,
+                   min_val: int=0, max_val: int=100, step: int=1, init_val: int=0) -> 'py_cui.controls.slider.SliderWidget':     
+                   
+                   
         """Function that adds a new label to the CUI grid
 
         Parameters
@@ -454,7 +464,7 @@ class WidgetSet:
             A reference to the created slider object.
         """
 
-        id = 'Widget{}'.format(len(self._widgets.keys()))
+        id = len(self._widgets.keys())
         new_slider = controls.slider.SliderWidget(id,
                                                   title,
                                                   self._grid,
@@ -470,6 +480,5 @@ class WidgetSet:
                                                   step,
                                                   init_val)
         self._widgets[id] = new_slider
-        self._logger.info('Adding widget {} w/ ID {} of type {}'
-                           .format(title, id, str(type(new_slider))))
+        self._logger.debug(f'Adding widget {title} w/ ID {id} of type {str(type(new_slider))}')
         return new_slider
